<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9548f1f81a43 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:51 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    permissionhandler
 * @desc    This class looks for the needed permissions to perform a single
  action. Actions are normaly submitted via we_cmds. There are two
  ways to use this class.
  a - get all needed permissions for a requested command.
  b - checks, if the user has the right to perform the requested action.
 *
 */
abstract class we_base_permission{
	public const USER_RESTRICTED = -1;
	public const WORKSPACE_HAS_USERS = -2;
	public const WORKSPACE_FOR_OBJECT = -3;
	public const SUBWORKSPACE_HAS_USERS = -4;
	public const WORKSPACE_USED_BY_OBJECTS = -5;
	public const NO_PERMISSION = -6;
	public const USED_BY_OTHER = -7;
	private const knownActions = [
		'switch_edit_page' => [
			we_base_constants::WE_EDITPAGE_PROPERTIES => [we_base_perm::WORK_PROPERTIES_CAN_SEE],
			we_base_constants::WE_EDITPAGE_INFO => [we_base_perm::WORK_INFOTAB_CAN_SEE],
			we_base_constants::WE_EDITPAGE_VALIDATION => [we_base_perm::WORK_VALIDATION_CAN_SEE],
		],
	];

	public static function hasPerm(array $perm, bool $or = true): bool{
		if(empty($_SESSION['user']['isWeSession'])){
			return false;
		}
		if((!empty($_SESSION['perms'][we_base_perm::ADMINISTRATOR]))){
			return true;
		}
		if(empty($perm)){
			return true;
		}
		$ret = !$or;
		foreach($perm as $cur){
			$ret = ($or ?
				$ret || (!empty($_SESSION['perms'][$cur])) :
				$ret && (!empty($_SESSION['perms'][$cur]))
				);
		}
		return $ret;
	}

	/**
	 * we_base_permission::getPermissionsForAction()
	 * @desc    This function looks in the array $knownActions for the needed
	  permissions for an action.
	  It returns either an array of permissions, or "none", when no
	  permission is needed for this action, or the action is not
	  listed.
	 *
	 * @param   string $requestedAction     the action the user wants to do (we_cmd[0])
	 * @param   int $parameter           a parameter to specify the action
	 *
	 * @return  string[]               of the needed Permissions
	 */
	static function getPermissionsForAction(string $requestedAction, int $parameter): array{

		/*
		  Here is the Array, which gives the connection between the action the user wants to make
		  and the needed permissions.
		  - first index is the action, the user wants to make
		  - the second is a paramter the action needs.
		  - the last is an array of strings containing all permissions, the user must have
		  each entry of the array reflects a condition, when one condition matches - the action is allowed.
		  - Several AND conditions in one conditionstring are seperated with ","

		  here are two example usages:

		  $knownActions["switch_edit_page"]["we_base_constants::WE_EDITPAGE_PROPERTIES"] = array("CAN_SEE_PROPERTIES");
		  action: switch editPage to property-page - needed right is "CAN_SEE_PROPERTIES"

		  $knownActions["another_action"]["another_para"] = array("Right1","Right2,Right3");
		  action: another_action with another_para - needed right can be: Right1 OR Right2 AND Right3

		 */
		//	The first entries are no we_cmd[0], but sometimes needed.
		return self::knownActions[$requestedAction][$parameter] ?? [];
	}

	/**
	 * we_base_permission::isUserAllowedForAction()
	 * @desc    When a user is allowed to do an action with a certain parameter,
	  true is returned, otherwise false.
	 *
	 * @see     we_base_permission::getPermissionsForAction
	 *
	 * @param   string $requestedAction     the action the user wants to do (we_cmd[0])
	 * @param   int $parameter           a parameter to specify the action
	 *
	 * @return  bool
	 */
	static function isUserAllowedForAction(string $requestedAction, int $parameter): bool{
		$neededPerm = self::getPermissionsForAction($requestedAction, $parameter);
		//  An array is returned, check the rights.
		if(empty($neededPerm)){
			//  no permissions are needed for this action
			return true;
		}
		return we_base_permission::hasPerm($neededPerm, false);
	}

	static function checkIfRestrictUserIsAllowed(int $id, string $table): bool{
		$row = getHash('SELECT CreatorID=' . we_users_user::getID() . ' AS isCreator,RestrictOwners,Owners,OwnersReadOnly FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		if(!$row || $row['isCreator'] || we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){ //	Owner or admin or file doesn't exist
			return true;
		}

		if($row['RestrictOwners']){ //	check which user - group has permission
			$userArray = array_filter(explode(',', $row['Owners']));

			$allowedGroup = false;

			//	check if usergroup is allowed
			foreach($_SESSION['user']['groups'] as $userGroup){
				if(in_array($userGroup, $userArray, true)){
					$allowedGroup = true;
					break;
				}
			}
			if(!in_array(we_users_user::getID(), $userArray) && !$allowedGroup){ //	user is no allowed user.
				return false;
			}

			//	user belongs to owners of document, check if he has only read access !!!
			if($row['OwnersReadOnly']){
				$arr = we_unserialize($row['OwnersReadOnly']);
				if(is_array($arr)){

					if(!empty($arr[we_users_user::getID()])){ //	if user is readonly user -> no delete
						return false;
					}
					if(in_array(we_users_user::getID(), $userArray)){ //	user NOT readonly and in restricted -> delete allowed
						return true;
					}

					//	check if group has rights to delete
					foreach($_SESSION['user']['groups'] as $userGroup){ //	user is directly in first group
						if(!empty($arr[$userGroup])){ //	group not allowed
							return false;
						}
						if(in_array($userGroup, $userArray)){ //	group is NOT readonly and in restricted -> delete allowed
							return true;
						}
					}
				}
			}
		}
		return true;
	}

	public static function noPermDialog(string $msg): void{
		echo we_html_tools::getHtmlTop(g_l('alert', '[no_perms_title]'), '', we_html_element::body(['class' => 'weDialogBody', 'onload' => "WE().layout.weEditorFrameController.getEditorFrame(window.name).setEditorIsLoading(false);"], we_html_tools::dialogLayout('<p class="defaultfont">' . $msg . '</p>', g_l('alert', '[no_perms_title]'))));
		exit();
	}

	/**
	 *
	 * @return array<string,array{title:string,defaults:array<string,int>}>
	 */
	public static function getDefaultPermissions(): array{
		return
			[
				'administrator' => [
					'title' => g_l('perms_administrator', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::ADMINISTRATOR => 0
					],
				],
				we_modules_info::USERS => [
					'title' => g_l('menu_moduleInformation', '[users][text]'),
					'defaults' => [/*
					  we_base_perm::USER_GROUP_NEW => 0,
					  we_base_perm::USER_NEW => 0,
					  we_base_perm::USER_GROUP_SAVE => 0,
					  we_base_perm::USER_SAVE => 0,
					  we_base_perm::USER_GROUP_DELETE => 0,
					  we_base_perm::USER_DELETE => 0, */
					]
				],
				'controlpermissions' => [
					'title' => g_l('perms_controlpermissions', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::WORK_DOCUMENT_SAVE_ALLOWED => 1,
						we_base_perm::WORK_CAN_PUBLISH => 1,
						we_base_perm::USER_CHANGE_DOCUMENT_OWNER => 0,
						we_base_perm::USER_CHANGE_DOCUMENT_PERMISSION => 0,
					]
				],
				'widget' => [
					'title' => g_l('perms_widget', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::WIDGET_EDIT_MFD => 0,
						we_base_perm::WIDGET_SHOW_USR => 1
					]
				],
				we_modules_info::CUSTOMER => [
					'title' => g_l('perms_customer', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::CUSTOMER) ? [
					we_base_perm::CUSTOMER_NEW => 0,
					we_base_perm::CUSTOMER_DELETE => 0,
					we_base_perm::CUSTOMER_EDIT => 0,
					we_base_perm::CUSTOMER__SHOW_ADMIN => 0,
					we_base_perm::CUSTOMER_AUTOLOGINID_VISIBLE => 0,
					we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER => 1,
					we_base_perm::CUSTOMER_CAN_CHANGE_DOCS => 1,
					we_base_perm::CUSTOMER_DECRYPT_FIELD => 1,
					we_base_perm::CUSTOMER_CHANGE_ENCRYPTED_FIELD => 1,
					] : [])
				],
				we_modules_info::GLOSSARY => [
					'title' => g_l('perms_glossary', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::GLOSSARY) ? [
					we_base_perm::GLOSSARY_NEW => 1,
					we_base_perm::GLOSSARY_EDIT => 1,
					we_base_perm::GLOSSARY_DELETE => 1,
					we_base_perm::GLOSSARY_EDIT_DICTIONARY => 1,
					] : [] )
				],
				'importExportpermissions' => [
					'title' => g_l('perms_importExportpermissions', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::EXIM_EXPORT => 0,
						we_base_perm::EXIM_EXPORTNODOWNLOAD => 0,
						we_base_perm::EXIM_IMPORT => 0,
						we_base_perm::EXIM_BACKUPLOG => 0,
						we_base_perm::EXIM_IMPORT_FILE => 1,
						we_base_perm::EXIM_IMPORT_SITE => 1,
						we_base_perm::EXIM_IMPORT_WXML => 0,
						we_base_perm::EXIM_IMPORT_GENERICXML => 0,
						we_base_perm::EXIM_IMPORT_CSV => 0,
						we_base_perm::EXIM_EXPORT_NEW => 0,
						we_base_perm::EXIM_EXPORT_DELETE => 0,
						we_base_perm::EXIM_EXPORT_EDIT => 0,
						we_base_perm::EXIM_EXPORT_MAKE => 0,
						we_base_perm::EXIM_EXPORT_GENERICXML => 1,
						we_base_perm::EXIM_EXPORT_CSV => 1
					]
				],
				we_modules_info::NAVIGATION => [
					'title' => g_l('perms_navigation', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::NAVIGATION_EDIT => 1,
						we_base_perm::NAVIGATION_DELETE => 1,
						we_base_perm::NAVIAGTION_EDIT_RULES => 0,
						we_base_perm::NAVIGATION_EDIT_DYNAMIC => 0,
					]
				],
				we_modules_info::NEWSLETTER => [
					'title' => g_l('perms_newsletter', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::NEWSLETTER) ? [
					we_base_perm::NEWSLETTER_NEW => 0,
					we_base_perm::NEWSLETTER_DELETE => 0,
					we_base_perm::NEWSLETTER_EDIT => 0,
					we_base_perm::NEWSLETTER_SEND => 0,
					we_base_perm::NEWSLETTER_SEND_TEST_EMAIL => 0,
					we_base_perm::NEWSLETTER_SETTINGS => 0,
					we_base_perm::NEWSLETTER_FILES => 0
					] : [])
				],
				we_modules_info::OBJECT => [
					'title' => g_l('perms_object', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::OBJECT) ? [
					we_base_perm::OBJECTFILE_NEW => 1,
					we_base_perm::OBJECT_NEW => 0,
					we_base_perm::OBJECTFILE_NEW_FOLDER => 1,
					we_base_perm::OBJECTFILE_DELETE => 1,
					we_base_perm::OBJECT_DELETE => 0,
					we_base_perm::OBJECTFILE_MOVE => 1,
					we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS => 1,
					we_base_perm::OBJECTS_CAN_COPY => 1
					] : [])
				],
				'rebuildpermissions' => [
					'title' => g_l('perms_rebuildpermissions', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::REBUILD => 1,
						we_base_perm::REBUILD_ALL => 1,
						we_base_perm::REBUILD_TEMPLATES => 1,
						we_base_perm::REBUILD_FILTERD => 1,
						we_base_perm::REBUILD_OBJECTS => 1,
						we_base_perm::REBUILD_INDEX => 1,
						we_base_perm::REBUILD_THUMBS => 1,
						//we_base_perm::REBUILD_NAVIGATION => 1,
						we_base_perm::REBUILD_META => 1
					]
				],
				we_modules_info::SCHEDULER => [
					'title' => g_l('perms_schedule', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::SCHEDULER) ? [
					we_base_perm::SCHEDULER_CAN_SEE => 1
					] : [])
				],
				'GUI' => [
					'title' => g_l('perms_SEEM', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::SEEM_CAN_SEE_MENUE => 1,
						we_base_perm::SEEM_CAN_WORK_NORMAL_MODE => 1,
						we_base_perm::SEEM_CHANGE_START_DOCUMENT => 1,
						we_base_perm::WORK_QUICKSTART_CAN_SEE => 1,
						we_base_perm::WORK_PROPERTIES_CAN_SEE => 1,
						we_base_perm::WORK_INFOTAB_CAN_SEE => 1,
						we_base_perm::WORK_VALIDATION_CAN_SEE => 1,
						we_base_perm::WORK_ACCESSIBLE_PARAMETERS_CAN_SEE => 1,
					]
				],
				'settings' => [
					'title' => g_l('moduleActivation', '[settings]'),
					'defaults' => [
						we_base_perm::USER_EDIT_SETTINGS => 1,
						we_base_perm::USER_EDIT_PASSWD => 1,
						we_base_perm::WORK_DOCTYPE_EDIT => 0,
						we_base_perm::WORK_CATEGORY_EDIT => 1,
						we_base_perm::WORK_FORMMAIL_SHOW => 0,
						we_base_perm::WORK_METADATAFIELD_EDIT => 0,
						we_base_perm::WORK_VALIDATION_EDIT => 0,
						we_base_perm::WORK_BROWSE_SERVER => 0,
					]
				],
				'tree' => [
					'title' => g_l('prefs', '[tree_title]'),
					'defaults' => [
						we_base_perm::WORK_DOCUMENTS_CAN_SEE => 1,
						we_base_perm::WORK_TEMPLATES_CAN_SEE => 0,
						we_base_perm::WORK_COLLECTIONS_CAN_SEE => 1,
						we_base_perm::OBJECTFILES_CAN_SEE => 1,
						we_base_perm::OBJECTS_CAN_SEE => 0,
					]
				],
				we_modules_info::SHOP => [
					'title' => g_l('perms_shop', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::SHOP) ? [
					we_base_perm::SHOP_NEW_ARTICLE => 0,
					we_base_perm::SHOP_DELETE_ARTICLE => 0,
					we_base_perm::SHOP_EDIT_ORDER => 0,
					we_base_perm::SHOP_DELETE_ORDER => 0,
					we_base_perm::SHOP_EDIT_PREFS => 0,
					we_base_perm::VARIANTS_CAN_EDIT => 1
					] : [])
				],
				'upload' => [
					'title' => g_l('prefs', '[upload][title]'),
					'defaults' => [
						we_base_perm::UPLOAD_DISABLE_SIZE_LIMIT => 0,
						we_base_perm::UPLOAD_DISABLE_DIM_LIMIT => 0,
					]
				],
				'versions' => [
					'title' => g_l('perms_versions', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::VERSIONS_CAN_SEE => 0,
						we_base_perm::VERSIONS_RESET => 0
					]
				],
				we_modules_info::VOTING => [
					'title' => g_l('perms_voting', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::VOTING) ? [
					we_base_perm::VOTING_NEW => 1,
					we_base_perm::VOTING_DELETE => 1,
					we_base_perm::VOTING_EDIT => 1
					] : [])
				],
				we_modules_info::WORKFLOW => [
					'title' => g_l('perms_workflow', '[perm_group_title]'),
					'defaults' => (we_modules_info::isActive(we_modules_info::WORKFLOW) ? [
					we_base_perm::WORKFLOW_NEW => 0,
					we_base_perm::WORKFLOW_DELETE => 0,
					we_base_perm::WORKFLOW_EDIT => 0,
					we_base_perm::WORKFLOW_EMPTY_LOG => 0
					] : [])
				],
				'workpermissions' => [
					'title' => g_l('perms_workpermissions', '[perm_group_title]'),
					'defaults' => [
						we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW => 1,
						we_base_perm::WORK_IMAGE_NEW => 1,
						we_base_perm::WORK_HTML_NEW => 1,
						we_base_perm::WORK_VIDEO_NEW => 1,
						we_base_perm::WORK_JS_NEW => 1,
						we_base_perm::WORK_CSS_NEW => 1,
						we_base_perm::WORK_TEXT_NEW => 1,
						we_base_perm::WORK_HTACCESS_NEW => 0,
						we_base_perm::WORK_OTHER_NEW => 1,
						we_base_perm::WORK_TEMPLATE_NEW => 0,
						we_base_perm::WORK_COLLECTION_NEW => 1,
						we_base_perm::WORK_DOCUMENT_FOLDER_NEW => 1,
						we_base_perm::WORK_COLLECTION_FOLDER_NEW => 1,
						we_base_perm::WORK_DOCUMENT_FOLDER_PATH_CHANGE => 0,
						we_base_perm::WORK_TEMPLATES_FOLDER_NEW => 0,
						we_base_perm::WORK_COLLECTION_SAVE => 1,
						we_base_perm::WORK_DOCUMENT_FOLDER_DELETE => 1,
						we_base_perm::WORK_COLLECTION_FOLDER_DELETE => 1,
						we_base_perm::WORK_TEMP_FOLDER_DELETE => 0,
						we_base_perm::WORK_DOCUMENT_DELETE => 1,
						we_base_perm::WORK_TEMPLATE_DELETE => 0,
						we_base_perm::WORK_COLLECTION_DELETE => 1,
						we_base_perm::WORK_DOCUMENT_MOVE => 1,
						we_base_perm::WORK_TEMPLATE_MOVE => 0,
						we_base_perm::WORK_COLLECTION_MOVE => 0,
						we_base_perm::WORK_DOCEXTENSION_EDIT => 0,
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES => 1,
						we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT => 1,
						we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED => 0,
						we_base_perm::WORK_FOLDERS_COPY => 0,
					]
				],
		];
	}

}
