<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4e4ff85098e9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-03 00:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*  a class for handling binary-documents like images. */

class we_document_binary extends we_document_files{
	/** @var string The HTML-Code which can be included in a HTML Document */
	protected $html = '';

	/** @var boolean Flag which indicates that the doc has changed!	 */
	public $DocChanged = false;

	/** @var object instance of metadata reader for accessing metadata functionality	 */
	private $metaDataReader = null;

	/** @var bool */
	protected $IsProtected = false;

	/** @var array for metadata read via $metaDataReader */
	var $metaData = [];

	function __construct(int $id = 0){
		parent::__construct();
		$this->initByID($id);
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'html' => we_base_request::STRING,
			'DocChanged' => we_base_request::BOOL,
			'IsProtected' => we_base_request::BOOL
		]);

		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO, we_base_constants::WE_EDITPAGE_CONTENT,
				we_base_constants::WE_EDITPAGE_VERSIONS]);
			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && (we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER, we_base_perm::CUSTOMER_CAN_CHANGE_DOCS]))){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
		}
	}

	/** must be called from the editor-script. Returns a filename which has to be included from the global-Script */
	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_IMAGEEDIT:
				return new we_editor_imageedit($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_binaryContent($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VERSIONS:
				return new we_editor_versions($this, $jsCmd);
		}
	}

	protected function getContentData(string $table): bool{
		$ret = parent::getContentData($table);
		$sitePath = $this->getSitePath();
		$realPath = $this->getRealPath();
		if(!file_exists($sitePath) && file_exists($realPath) && !is_dir($realPath)){
			we_base_file::makeHardLink($realPath, $sitePath);
		}
		if(file_exists($sitePath) && filesize($sitePath)){
			$this->setElementS('data', $sitePath, 'image');
		}
		return $ret;
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$this->issetElement('data')){
			$this->getContentData($this->Table);
		}
		if($this->getFilesize() == 0){
			$this->errMsg = g_l('metadata', '[file_size_0]');
			return false;
		}
		if(parent::we_save($resave, $skipHook)){
			$this->DocChanged = false;
			$this->setElementS('data', $this->getSitePath());
			$this->writeMetaValues();
			return $this->insertAtIndex();
		}

		return false;
	}

	public function we_publish(): bool{
		return $this->we_save();
	}

	function getDocumentData(string $path = '', int $size = -1): string{
		$file = $this->getElementS('data');
		return ($file && file_exists($file) ?
			($size == -1 ?
			we_base_file::load($file) :
			we_base_file::loadPart($file, 0, $size)
			) :
			'');
	}

	protected function writeDocument(): bool{
		$file = $this->getElementS('data');
		if(!($file && file_exists($file)) ||
			$this->isPathNotValid()
		){
			return false;
		}
		$sitePath = $this->getSitePath();
		$realPath = $this->getRealPath();
		if(!we_base_file::checkAndMakeFolder(dirname($sitePath), true) ||
			!we_base_file::checkAndMakeFolder(dirname($realPath), true) ||
			($file != $sitePath && !we_base_file::copyFile($file, $sitePath))){
			return false;
		}
		if(!we_base_file::makeHardLink($file, $realPath)){
			return false;
		}
		if($this->isMoved()){
			we_base_file::delete($this->getRealPath(true));
			we_base_file::delete($this->getSitePath(true));
			$this->rewriteNavigation();
		}
		$this->update_filehash();

		return true;
	}

	public function uploadImportFile(array $fileVars, array $docVars): void{
		$this->IsSearchable = $docVars['isSearchable'] ?? $this->IsSearchable;
		parent::uploadImportFile($fileVars, $docVars);
	}

	public function uploadSetData(array $fileVars): bool{
		if(($fh = @fopen(($tempFile = WE_DOCUMENT_ROOT . $fileVars['fileTemp']), 'rb'))){
			$this->setElementS('data', $tempFile);
			fclose($fh);
			return true;
		}

		return false;
	}

	protected function writeSiteDir(string $content): bool{
		//do nothing - remove functionality added
		return true;
	}

	protected function writeMainDir(string $content): bool{
		//do nothing - remove functionality added
		return true;
	}

	/** gets the filesize of the document */
	function getFilesize(): int{
		$file = $this->getElementS('data');
		return (file_exists($file) ? filesize($file) ?: 0 : 0);
	}

	function insertAtIndex(array $only = [], array $fieldTypes = []): bool{
		if(!(!empty($this->IsSearchable) && $this->Publish_Date)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));
			return true;
		}

		$text = '';
		foreach($this->nextElement('txt')as $k){
			$dat = $this->getElementS((string) $k);
			$foo = we_base_util::isSerialized($dat) ? we_unserialize($dat) : '';
			$text .= (!is_array($foo) && $dat && $k !== 'data' ? ' ' . $dat : '');
		}
		$text = self::formatIndexText($text);
		$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID) . ($text ? ' AND WorkspaceID!=' . intval($this->ParentID) : ''));
		if(!$text){
			return true;
		}

		return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
					'ID' => $this->ID,
					'Text' => $text,
					'WorkspaceID' => intval($this->ParentID),
					'Category' => $this->Category ?: null,
					'Title' => self::formatIndexText($this->getElementS(we_contents_root::TITLE_FIELD)) ?: null,
					'Description' => self::formatIndexText($this->getElementS(we_contents_root::DESCRIPTION_FIELD)) ?: null,
					'Keywords' => self::formatIndexText($this->getElementS(self::KEYWORD_FIELD)) ?: null,
		]));
	}

	public function we_new(): void{
		parent::we_new();
		$this->Filename = $this->getDefaultFilename();
	}

	/**
	 * create instance of weMetaData to access metadata functionality:
	 */
	protected function getMetaDataReader(bool $force = false): we_metadata_metaData{
		if(!$force){
			throw new we_exception_base('not available');
		}
		if(!$this->metaDataReader){
			$source = $this->getElementS('data');
			if(file_exists($source)){
				$this->metaDataReader = new we_metadata_metaData($source);
			} else {
				throw new we_exception_file('source file not found');
			}
		}
		return $this->metaDataReader;
	}

	/**
	 * @abstract tries to read ebmedded metadata from file
	 * @return array|false false if either no metadata is available or something went wrong
	 */
	function getMetaData(): array{
		try{
			$reader = $this->getMetaDataReader();
			$this->metaData = $reader->getMetaData();
			return $this->metaData;
		} catch (we_exception_base $e){
			return [];
		}
	}

	protected function setElementsFromHTTP(): void{
		// preventing fields from override
		if(we_base_request::string('we_cmd', '', 0) === 'update_file'){
			return;
		}
		parent::setElementsFromHTTP();
	}

	/**
	 * returns HTML code for embedded metadata of current image with custom form fields
	 */
	function formMetaData(): string{
		// first we fetch all defined metadata fields from tblMetadata:
		$defined_fields = we_metadata_metaData::getDefinedMetaDataFields();

		// show an alert if there are none
		if(empty($defined_fields)){
			return '';
		}

		// second we build all input fields for them and take
		// the elements of this imageDocument as values:
		$fieldcount = count($defined_fields);
		$content = new we_html_table(['class' => 'default withSpace', 'style' => "margin-top:4px;"], $fieldcount, 1);
		$mdcontent = '';
		$i = 0;
		foreach($defined_fields as $field){
			$tagName = $field['tag'];
			switch($tagName){
				case self::TITLE_FIELD:
				case self::DESCRIPTION_FIELD:
				case self::KEYWORD_FIELD:
					continue 2;
				default:
					$type = $field['type'];

					switch($type){
						case 'textarea':
						case 'wysiwyg':
							$inp = $this->formTextArea('txt', $tagName, $tagName, 10, 30, ['onchange' => '_EditorFrame.setEditorIsHot(true);', 'style' => 'width:500px;height:150px;border: #AAAAAA solid 1px']);
							break;
						case 'date':
							$inp = we_html_tools::formElementTable($tagName, we_html_tools::getNewDateInput('we_' . $this->Name . '_date[' . $tagName . '][%s]', new DateTime(date('Y-m-d H:i:s', $this->getElementI($tagName))), we_html_tools::FORMAT_DATE_TIME, true));
							break;
						case 'textfield':
						default:
							$inp = $this->formMetaField($tagName);
					}

					$content->setCol($i, 0, $inp);
			}
			$i++;
		}

		$mdcontent .= $content->getHtml();

		// Return HTML
		return $mdcontent;
	}

	/**
	 * Returns HTML code for Upload Button and infotext
	 */
	function formUpload(): we_fileupload_ui_wedoc{
		$isempty = empty(($size = $this->getFilesize()));
		$size = g_l('metadata', '[filesize]') . ": " . round(($size / 1024), 2) . "&nbsp;KB";
		$ext = $isempty ? '' : $this->Extension;
		$typetext = g_l('metadata', '[filetype]') . ': ' . ($ext ? substr($ext, 1) : '');

		$metaData = $this->getMetaData();
		$mdtypes = [];
		if($metaData){
			if(!empty($metaData['exif'])){
				$mdtypes[] = 'Exif';
			}
			if(!empty($metaData['iptc'])){
				$mdtypes[] = 'IPTC';
			}
			if(!empty($metaData['pdf'])){
				$mdtypes[] = 'PDF';
			}
		}
		$metatext = (we_editor_functions::isSeeMode() ? '' :
			g_l('metadata', '[supported_types]') . ': ' .
			'<a href="javascript:parent.frames.editHeader.weTabs.setActiveTab(\'tab_2\');we_cmd(\'switch_edit_page\',2,\'' . we_base_WE::$transaction . '\');">' .
			(count($mdtypes) > 0 ? implode(', ', $mdtypes) : g_l('metadata', '[none]')) .
			'</a>');

		$fileinfo = [
			'size' => $size,
			'ct' => $this->ContentType,
			'ext' => $ext,
			'typetext' => $typetext,
			'metatext' => $metatext,
			'thumbnail' => $this->getThumbnail(100, 100),
			'isempty' => $isempty
		];
		return new we_fileupload_ui_wedoc('we_File', [$this->ContentType], [], [], [], $fileinfo);
	}

	protected function getThumbnail(int $width = 150, int $height = 100): string{
		return '';
	}

	function savebinarydata(): void{
		$data = $this->getElementS('data');
		if($data && (strlen($data) > 512 || !@file_exists($data))){ //assume data>512 = binary data
			$path = we_base_file::saveTemp($data, '', 3600);
			$this->setElementS('data', $path);
		}
	}

	public function formProperties(): string{
		return '';
	}

	function formIsProtected(): string{
		return we_html_forms::checkboxWithHidden((bool) $this->IsProtected, 'we_' . $this->Name . '_IsProtected', g_l('weClass', '[protected]'), false, '', '_EditorFrame.setEditorIsHot(true);');
	}

	function formReferences(we_base_jsCmd $jsCmd): array{
		$search = new we_search_search(new we_search_view(new we_search_model()));
		$search->searchMediaLinks(0, true, [$this->ID]);
		$ml = $search->getUsedMediaLinks();
		$accessibles = $ml['accessible']['mediaID_' . $this->ID] ?? [];
		$notaccessibles = $ml['notaccessible']['mediaID_' . $this->ID] ?? [];
		$groups = $ml['groups']['mediaID_' . $this->ID] ?? [];

		if(empty($groups)){
			return ['form' => g_l('weClass', '[notReferenced]'), 'num' => 0];
		}

		$ref = $values = [];
		$c = $num = 0;
		$limit = 20;
		foreach($groups as $group){
			$cna = isset($notaccessibles[$group]) && is_array($notaccessibles[$group]) ? count($notaccessibles[$group]) : 0;
			$ca = (isset($accessibles[$group]) && is_array($accessibles[$group]) ? count($accessibles[$group]) : 0) + $cna;
			$num += $ca;

			$values[$group . ' (' . ($ca) . ($cna ? ', davon ' . $cna . ' ' . g_l('weClass', '[medialinks_unaccessible]') . '' : '') . ')'] = we_html_tools::OPTGROUP;
			$cc = 0;
			if(empty($accessibles[$group])){
				continue;
			}
			foreach($accessibles[$group] as $v){
				if($cc++ >= $limit){
					$values[-1] = '[ + ' . (count($accessibles[$group]) - $limit) . ' ' . g_l('weClass', '[medialinks_more]') . ' ]';
					break;
				}
				$element = preg_replace('|NN[0-9]\]+$|', 'NN]', $v['element'] ?? '');
				$values[++$c] = ($v['path'] ?? '') . ($element ? ', in: ' . $element : '');
				$ref['id_' . $c] = $v;
			}
		}
		$button = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.we_openMediaReference(window, document.we_form.MediaReferences.value);");
		$jsCmd->addCmd('setMediaReferences', $ref);
		$form = we_html_tools::formElementTable('', we_html_tools::select('MediaReferences', $values, 1, '', false, [], true, 388), $button);

		return ['form' => $form, 'num' => $num];
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[document]'), $this->formIsSearchable() . $this->formIsProtected(), we_html_multiIconBox::PROP_DOC)))
					->add(we_modules_info::isActive(we_modules_info::CATEGORY) ? (new we_html_multiIconBoxContentLine(g_l('weClass', '[category]'), $this->formCategory($jsCmd), we_html_multiIconBox::PROP_CATEGORIES))->fold() : null)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
					, 'PropertyPage'))
				->marginLeft()
				->get();
	}

	public static function showBinaryDoc(): void{
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 2);
		$we_doc = we_contents_root::getSessionDocument($we_transaction);
		assert($we_doc instanceof we_document_binary);
		switch(we_base_request::string('we_cmd', '', 1)){
			case we_base_ContentTypes::IMAGE:
				$contenttype = $we_doc->getElementS('type');
				break;
			case we_base_ContentTypes::VIDEO:
				$contenttype = 'video/' . str_replace('.', '', $we_doc->Extension);
				break;
			case we_base_ContentTypes::AUDIO:
				$contenttype = 'audio/' . str_replace('.', '', $we_doc->Extension);
				break;
			case we_base_ContentTypes::PDF:
				$contenttype = 'application/pdf';
				break;
			case we_base_ContentTypes::APPLICATION:
				switch($we_doc->Extension){
					case '.zip':
						$contenttype = 'application/zip';
						break;
					case '.doc':
						$contenttype = 'application/msword';
						break;
					case '.xls':
						$contenttype = 'application/vnd.ms-excel';
						break;
					case '.ppt':
						$contenttype = 'application/vnd.ms-powerpoint';
						break;
					case '.sit':
						$contenttype = 'application/x-stuffit';
						break;
					case '.hqx':
						$contenttype = 'application/mac-binhex40';
						break;
					default:
						$contenttype = 'application/octet-stream';
				}
				break;
			default:
				exit('unsupported request');
		}
		header('Content-disposition: filename=' . $we_doc->Filename . $we_doc->Extension);
		header('Content-Type: ' . $contenttype);
		header('Pragma: no-cache');
		header('Expires: 0');

		$dataPath = $we_doc->getElementS('data');
		session_write_close();
		if(($tid = we_base_request::int('we_cmd', 0, 3))){ // create thumbnail
			$thumbObj = new we_contents_thumbnail();
			$thumbObj->initByThumbID($tid, $we_doc->ID, $we_doc->Filename, $we_doc->Path, $we_doc->Extension, $we_doc->getElementI('origwidth'), $we_doc->getElementI('origheight'), $we_doc->getDocument());
			readfile($thumbObj->getThumb());
			exit();
		}
		readfile($dataPath);
		exit();
	}

	public function mustCheckEditPageArray(): bool{
		return true;
	}

	function getHtml(bool $dyn = false): string{
		//overwrite
		return '';
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_document_textContent::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(): void{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return;
		}
		$this->documentCustomerFilter = we_customer_documentFilter::getFilterByIdAndTable($this->ID, $this->Table);
	}

}
