<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4805498ffe86 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-10-29 13:52 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class taskFragment()
 *
 * This class offers methods to split tasks in more than one fragment.
 * It is needed if you need to do a lot of work, which takes time
 * longer than the timeout of some servers
 */
abstract class we_fragment_base{
	/**
	 * Number of all tasks.
	 * @var        int
	 */
	protected $numberOfTasks = 1;

	/**
	 * Number of current tasks.
	 * @var        int
	 */
	protected $currentTask = 0;

	/**
	 * Number of tasks per fragment.
	 * @var        int
	 */
	protected $taskPerFragment = 1;

	/**
	 * Array of the data.
	 * @var        array
	 */
	protected $alldata = [];

	/**
	 * Data for the current task.
	 * @var        mixed
	 */
	protected $data = null;

	/**
	 * Name for the whole fragment action.
	 * This variable is used for a reference, so it must be unique
	 * @var        string
	 */
	protected $name;
	protected $initdata = null;
	protected $jsCmd = null;

	/**
	 * init Data.
	 * @var        array
	 */

	/**
	 * This is the constructor of the class. Everything will be done here.
	 *
	 * @param      string $name
	 * @param      int $taskPerFragment
	 * @param      array<string,string> $bodyAttributes
	 * @param      array $initdata
	 */
	public function __construct(string $name, int $taskPerFragment, array $bodyAttributes = [], array $initdata = [], we_base_jsCmd $jsCmd = null){
		$this->name = $name;
		$this->taskPerFragment = $taskPerFragment;
		if($initdata){
			$this->initdata = $initdata;
		}
		$this->jsCmd = $jsCmd ?: new we_base_jsCmd();

		//FIXME: make this DB entries; create method for early creation, since the whole data might be too much for memory!
		$filename = WE_FRAGMENT_PATH . '/' . $this->name;
		$this->currentTask = we_base_request::int('fr_' . $this->name . '_ct', 0);
		if(file_exists($filename) && $this->currentTask){
			$ser = we_base_file::load($filename);
			if(!$ser){
				exit('Could not read: ' . $filename);
			}
			$this->alldata = we_unserialize($ser);
		} else {
			$this->taskPerFragment = $taskPerFragment;
			$this->init();
			if(!we_base_file::save($filename, we_serialize($this->alldata, SERIALIZE_PHP))){
				exit('Could not write: ' . $filename);
			}
			we_base_file::insertIntoCleanUp($filename, 10 * 3600);
		}
		$this->numberOfTasks = count($this->alldata);
	}

	public function run(): void{
		$this->updateTaskPerFragment();
		for($i = 0; $i < $this->taskPerFragment && $this->currentTask < $this->numberOfTasks; $i++){
			$this->data = $this->alldata[$this->currentTask];
			$this->doTask();
			$this->currentTask++;
		}

		$this->printPage([]);
	}

	protected function printPage(array $bodyAttributes = []): void{
		$this->updateProgressBar();
		if($this->currentTask >= $this->numberOfTasks){
			$filename = WE_FRAGMENT_PATH . '/' . $this->name;
			we_base_file::delete($filename);
			$this->finish();
		} else {
			$this->getJSReload();
		}

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'we_fragment.js') . $this->jsCmd->getCmds(), we_html_element::body($bodyAttributes));
	}

	protected function getJSReload(): void{
		$nextTask = $this->currentTask;

		if(($nextTask < $this->numberOfTasks)){
			$tmp = $_REQUEST;
			$tmp['fr_' . $this->name . '_ct'] = $nextTask;
			$tmp['doFragments'] = 1;
			$tail = http_build_query($tmp, '', '&', PHP_QUERY_RFC3986);
			$this->jsCmd->addCmd('location', ['doc' => 'document', 'loc' => WEBEDITION_DIR . 'we_cmd.php?' . $tail]);
		}
	}

	abstract protected function updateProgressBar(): void;

	/**
	 * Overwrite this function to do the work for each task.
	 *
	 */
	abstract protected function doTask(): void;

	protected function updateTaskPerFragment(): void{

	}

	/**
	 * Overwrite this function to initialize the array taskFragment::alldata.
	 *
	 */
	protected function init(): void{
		$this->alldata = $this->initdata;
	}

	/**
	 * Overwrite this function to do the work when everything is finished.
	 *
	 */
	protected function finish(): void{

	}

}
