<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1ba31a7e23d9 $
 * $Author: Lukas Imhof $
 * $Date: 2020-07-09 16:40 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of WebEdition Newsletter Block
 *
 */
class we_newsletter_group extends we_newsletter_base{
	public const FILTER_FIELDS = ['fieldname', 'operator', 'fieldvalue', 'logic', 'hours', 'minutes'];

	/** @var int */
	protected $NewsletterID = 0;

	/** @var string */
	var $Emails = '';

	/** @var string[] */
	protected $Extern = [];

	/** @var int[] */
	protected $Customers = [];

	/** @var bool */
	protected $SendAll = true;

	/** @var array<int,array<string,mixed>> */
	public $Filter = []; //due to processVariables

	/** array<string,string> */
	protected $settings;

	/** @var array<string,string> */
	var $persistent_slots = [
		'NewsletterID' => we_base_request::INT,
		'Emails' => we_base_request::RAW_CHECKED,
		'Extern' => we_base_request::WEFILELISTA,
		'Customers' => we_base_request::INTLISTA,
		'SendAll' => we_base_request::BOOL,
		'Filter' => we_base_request::SERIALIZED_JSON
	];

	public function __construct(int $groupID = 0){
		parent::__construct(NEWSLETTER_GROUP_TABLE);
		$this->settings = self::getSettings();
		$tmp = $this->settings['global_mailing_list'] ?? '';
		$this->Extern = $tmp ? we_base_request::filterVar($tmp, we_base_request::WEFILELISTA) : []; //@phan-suppress-current-line PhanDeprecatedFunction

		if($groupID){
			$this->load($groupID);
		}
	}

	/*	 * **************************************
	 * save mailing list to database
	 *
	 * *************************************** */

	public function preserveFilter(): void{
		//make some fields up to date
		foreach($this->Filter as &$v){
			switch($v['fieldname'] ?? ''){
				case 'MemberSince_Date':
				case 'LastAccess_Date':
				case 'LastLogin_Date':
					if(!empty($v['fieldvalue'])){
						if(stristr($v['fieldvalue'], '.')){
							$date = explode('.', $v['fieldvalue']);
							$v['fieldvalue'] = new DateTime(sprintf('%04d-%02d-%02d %02d:%02d:00', intval($date[2]), intval($date[1]), intval($date[0]), intval($v['hours']), intval($v['minutes'])));
						}
					}
			}
		}
	}

	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		$this->preserveFilter();
		return parent::save();
	}

	/**
	 *
	 * @return array<int,array<string,mixed>>
	 */
	function getFilter(): array{
		return $this->Filter;
	}

	function delete(): bool{
		return parent::delete();
	}

	function checkEmails(): void{
		$mails = [];
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			if(empty($this->settings['customer_email_field'])){
				t_e('empty setting for customer email field');
			} elseif($this->Customers){
				$mails = $this->db->getAllq('SELECT ' . $this->settings['customer_email_field'] . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . implode(',', $this->Customers) . ')', true);
			}
		}

		$emails = array_merge($mails, we_newsletter_base::getEmailsFromList($this->Emails), we_newsletter_newsletter::getEmailsFromExtern($this->Extern));
		'@phan-var string[] $emails';
		foreach($emails as $email){
			'@phan-var string $email';
			if(!we_check_email($email)){
				throw new we_exception_newsletter($email, we_newsletter_newsletter::MALFORMED_GROUP);
			}
		}
	}

	function addFilter(string $name = '', int $pos = 0): void{
		$arr = $pos > 0 ? array_slice($this->Filter, 0, $pos) : [];
		$arr[] = ['fieldname' => $name, 'operator' => 0, 'fieldvalue' => '', 'logic' => '', 'hours' => '', 'minutes' => ''];
		$this->Filter = array_merge($arr, array_slice($this->Filter, $pos));
	}

	/**
	 *
	 * @param array<string,mixed> $filter
	 * @return void
	 */
	function appendFilter(array $filter): void{
		$this->Filter[] = $filter;
	}

	function delFilter(int $pos): void{
		unset($this->Filter[$pos]);
		$this->Filter = array_values($this->Filter);
	}

	function delallFilter(): void{
		$this->Filter = [];
	}

//---------------------------------- STATIC FUNCTIONS -------------------------------

	/**
	 * return all newsletter blocks for given newsletter id
	 *
	 * @param int $newsletterID
	 * @return we_newsletter_group[]
	 */
	static function getAllGroups(int $newsletterID): array{
		$ids = DB_WE::$inst->getAllq('SELECT ID FROM ' . NEWSLETTER_GROUP_TABLE . ' WHERE NewsletterID=' . $newsletterID . ' ORDER BY ID', true);
		$ret = [];
		foreach($ids as $id){
			$ret[] = new self((int) $id);
		}
		return $ret;
	}

	/**
	 * @return array<string,string>
	 */
	static function getSettings(): array{
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return DB_WE::$inst->getAllFirstq('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter"', false);
	}

	/**
	 * @param int[] $cust
	 * @return void
	 */
	public function addCustomer(array $cust): void{
		$this->Customers = array_unique(array_merge($this->Customers, $cust));
	}

	public function delCustomer(int $cust): void{
		foreach($this->Customers as $k => $v){
			if($v == $cust){
				unset($this->Customers[$k]);
			}
		}
	}

	public function delAllCust(): void{
		$this->Customers = [];
	}

	/**
	 * @return int[]
	 */
	public function getCustomers(): array{
		return $this->Customers;
	}

	public function setNewsletterID(int $id): void{
		$this->NewsletterID = $id;
	}

	public function isSendAll(): bool{
		return $this->SendAll;
	}

	/**
	 * @return string[]
	 */
	public function getExtern(): array{
		return $this->Extern;
	}

	/**
	 * @param string[] $ext
	 * @return void
	 */
	public function setExtern(array $ext): void{
		$this->Extern = $ext;
	}

	public function getHidden(int $counter): string{
		$vals = [];
		foreach($this->persistent_slots as $per => $type){
			$vals['group' . $counter . '_' . $per] = $this->convertToHiddenString($per, $type, $this->$per);
		}
		return we_html_element::hiddens($vals);
	}

	public function getHiddenFilter(int $counter): string{
		$filter = $this->getFilter();
		if(!$filter){
			return '';
		}
		$vals = ['filter_' . $counter => count($filter)];

		foreach($filter as $k => $v){
			foreach(self::FILTER_FIELDS as $field){
				if(isset($v[$field])){
					$vals['filter_' . $field . '_' . $counter . '_' . $k] = $v[$field];
				}
			}
		}
		return we_html_element::hiddens($vals);
	}

	public function setFromRequest(int $id, int $gkey): void{
		$this->setNewsletterID($id);

		foreach($this->persistent_slots as $per => $type){
			$this->$per = we_base_request::_($type, 'group' . $gkey . '_' . $per, $this->$per);
		}

		// Filter
		$count = (isset($_REQUEST['filter_' . $gkey]) ? $_REQUEST['filter_' . $gkey]++ : 0);
		if($count){
			$this->Filter = [];
			for($i = 0; $i < $count; $i++){
				$new = [];

				foreach(self::FILTER_FIELDS as $field){
					if(($tmp = we_base_request::_(we_base_request::RAW_CHECKED, 'filter_' . $field . '_' . $gkey . '_' . $i, false)) !== false){
						$new[$field] = $tmp;
					}
				}

				if($new){
					$this->appendFilter($new);
					$this->preserveFilter();
				}
			}
		}
	}

}
