<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * This function writes the shop data (order) to the database
 */
abstract class we_tag_writeShopData extends we_weTag_tag{
	/** @var ?bool */
	public static $status = null;

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::$status = false;
		$name = (string) self::getAttribute('name', $attribs, '', we_base_request::STRING);
		self::attributeMissing($attribs, ['pricename' => false], __CLASS__);
		if(!$name){
			self::attributeMissing($attribs, ['shopname' => false], __CLASS__);
		}

		$shopname = (string) self::getAttribute('shopname', $attribs, '', we_base_request::STRING) ?: $name;
		$pricename = self::getAttribute('pricename', $attribs, '', we_base_request::STRING);
		$shipping = self::getAttribute('shipping', $attribs, '', we_base_request::FLOAT);
		$shippingIsNet = self::getAttribute('shippingisnet', $attribs, false, we_base_request::BOOL);
		$shippingVatRate = self::getAttribute('shippingvatrate', $attribs, 0, we_base_request::FLOAT);
		$netprices = self::getAttribute('netprices', $attribs, true, we_base_request::BOOL);
		$useVat = self::getAttribute('usevat', $attribs, false, we_base_request::BOOL);
		$customPrefix = self::getAttribute('customPrefix', $attribs, '', we_base_request::STRING);
		$customPostfix = self::getAttribute('customPostfix', $attribs, '', we_base_request::STRING);

		$customer = ($_SESSION['webuser'] ?? []);
		unset($customer['Password'], $customer['_Password'], $customer['ID'], $customer['Username'], $customer['LoginDenied'], $customer['MemberSince_Date'], $customer['LastLogin_Date'], $customer['LastAccess_Date'], $customer['AutoLoginDenied'], $customer['AutoLogin'], $customer['Mod_Date'], $customer['ModifiedBy'], $customer['Path'], $customer['Newsletter_Ok'], $customer['registered'], $customer['AutoLoginID']
		);

		if($useVat){
			$weShopVatRule = we_shop_vatRule::getShopVatRule();
			$calcVat = $weShopVatRule->executeVatRule($customer);
		} else {
			$calcVat = false;
		}

		try{// Check for Shop being set
			$basket = we_tag_createShop::getBasket($shopname);
		} catch (Exception $ex){
			self::$status = true;
			return '';
		}
		$shoppingItems = $basket->getShoppingItems();
		$cartFields = $basket->getCartFields();

		if(empty($shoppingItems)){
			self::$status = true;
			return '';
		}

		$DB_WE = DB_WE::$inst;
		$doc = self::getDocForTag('top');
		$lang = substr($doc->Language, 0, 2);
		//first insert essential order data
		$DB_WE->query('INSERT INTO ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
				'shopname' => $shopname,
				'customerID' => intval($_SESSION['webuser']['ID']),
				'customerData' => we_serialize($customer, SERIALIZE_JSON, false, 5, true),
				'customerLang' => $lang,
				'customFields' => $cartFields ? we_serialize($cartFields, SERIALIZE_JSON, false, 0, true) : null,
				'pricesNet' => $netprices,
				'priceName' => $pricename,
				'shippingCost' => $shipping, //we prefill this
				'shippingNet' => $shippingIsNet,
				'shippingVat' => $shippingVatRate,
				'calcVat' => $useVat ? $calcVat : 0,
		]));

		$orderID = $DB_WE->getInsertId();
		$basket->setOrderID($orderID);

		if($customPostfix || $customPrefix){ //update customOrderNo
			$DB_WE->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
					'customOrderNo' => $customPrefix . $orderID . $customPostfix
				]) . ' WHERE ID=' . $orderID);
		}

		$totPrice = 0;
		$categoryMode = we_shop_category::isCategoryMode();

		foreach($shoppingItems as $shoppingItem){
			$preis = we_base_util::std_numberformat($shoppingItem['serial']['we_' . $pricename] ?? $shoppingItem['serial'][$pricename]);
			$totPrice += $preis * $shoppingItem['quantity'];

			// foreach article we must determine the correct tax-rate
			if($categoryMode){
				$wedocCategory = $shoppingItem['serial'][we_listview_base::CATEGORY_FIELD];
				$billingCountry = we_shop_category::getCountryFromCustomer(false, $_SESSION['webuser']);
				$catId = !empty($shoppingItem['serial'][we_shop_shop::CATEGORY_FIELD_NAME]) ? $shoppingItem['serial'][we_shop_shop::CATEGORY_FIELD_NAME] : 0;

				try{
					$shopVat = we_shop_category::getShopVatByIdAndCountry($catId, $wedocCategory, $billingCountry)->vat;
				} catch (Throwable $e){
					$shopVat = null;
				}
				$shopCategory = $catId;
			} else {
				$vatId = $shoppingItem['serial'][we_shop_shop::VAT_FIELD_NAME] ?? 0;
				$shopVat = we_shop_vats::getVatRateForSite($vatId, 0);
				$shopCategory = 0;
			}

			$dat = $shoppingItem['serial'];

			$docid = intval($shoppingItem['id']);
			$pub = $dat[we_listview_base::PROPPREFIX . 'published'];
			$type = (!empty($dat[we_listview_base::PROPPREFIX . 'contenttype'] && $dat[we_listview_base::PROPPREFIX . 'contenttype'] === we_base_ContentTypes::OBJECT_FILE) ? 'object' : 'document');
			$variant = $dat[we_listview_base::VARIANT_FIELD];

			$orderDocID = fI('SELECT ID FROM ' . SHOP_ORDER_DOCUMENT_TABLE . ' WHERE DocID=' . $docid . ' AND type="' . $type . '" AND variant="' . $DB_WE->escape($variant) . '" AND Published=' . $DB_WE->escape($pub));
			if(!$orderDocID){
				$data = $dat;
				unset($data['we_shoptitle'], $data['we_shopdescription'], $data[we_shop_shop::ARTICLE_CUSTOM_FIELD], $data['shopvat'], $data[we_shop_shop::CATEGORY_FIELD_NAME], $data[we_listview_base::VARIANT_FIELD]);
				//add document first
				$DB_WE->query('INSERT INTO ' . SHOP_ORDER_DOCUMENT_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocID' => $docid,
						'type' => $type,
						'variant' => $variant,
						'Published' => $pub,
						'title' => strip_tags($dat['shoptitle']),
						'description' => strip_tags($dat['shopdescription']),
						'CategoryID' => $shopCategory ?: 0,
						'SerializedData' => we_serialize($data, SERIALIZE_JSON, false, 5, true)
				]));
				$orderDocID = $DB_WE->getInsertId();
			}

			if(!$DB_WE->query('INSERT INTO ' . SHOP_ORDER_ITEM_TABLE . ' SET ' . we_database_base::arraySetter([
						'orderID' => $orderID,
						'orderDocID' => $orderDocID,
						'quantity' => abs($shoppingItem['quantity']),
						'Price' => $preis,
						'Vat' => $shopVat,
						'customFields' => $shoppingItem['serial'][we_shop_shop::ARTICLE_CUSTOM_FIELD] ? we_serialize($shoppingItem['serial'][we_shop_shop::ARTICLE_CUSTOM_FIELD], SERIALIZE_JSON, false, 0, true) : null,
				]))){

				t_e('error during write shop data contents of basket', $shoppingItems);
				self::$status = true;
				return 'Data Insert Failed';
			}
		}

		if($shipping === ''){// we have to change shipping costs
			$weShippingControl = we_shop_shippingControl::getShippingControl();

			$DB_WE->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
					'shippingCost' => $weShippingControl->getShippingCostByOrderValue($totPrice, $customer),
					'shippingNet' => (bool) $weShippingControl->isNet,
					'shippingVat' => (float) $weShippingControl->vatRate,
				]) . ' WHERE ID=' . $orderID
			);
		}


		$weShopStatusMails = we_shop_statusMails::getShopStatusMails();
		$weShopStatusMails->checkAutoMailAndSend('Order', $orderID, $customer, $lang);
		return '';
	}

}
